
#[macro_export]
macro_rules! point_from_string {
    ( $t:ty ) => {
        impl TryFrom<String> for Point<$t> {
            type Error = anyhow::Error;
            fn try_from(value: String) -> Result<Self, Self::Error> {
                if !value.contains("(") || !value.contains(")") || !value.contains(",") {
                    anyhow::bail!("malformed point, expected \"(x, y)\", got {:?}", value);
                }
                let mut work = value.clone().trim().to_string();
                work = work.replace("(", "").replace(")", "").replace(" ", "");
                if let Some(parts) = work.split_once(",") {
                    let r = parts.0.parse::<$t>();
                    match r {
                        Ok(x) => {
                            let r2 = parts.1.parse::<$t>();
                            match r2 {
                                Ok(y) => {
                                    return Ok(Point{x, y})
                                }
                                Err(e) => {
                                    return Err(anyhow::format_err!("malformed y value, {}", e));
                                }
                            }
                        },
                        Err(e) => {
                            return Err(anyhow::format_err!("malformed x value, {}", e));
                        }
                    }
                } else {
                    anyhow::bail!("malformed point, expected \"(x, y)\", got {:?}", value);
                }
            }
        }
    };
}

#[macro_export]
macro_rules! point_distance_value {
    ( $t:ty ) => {
        impl Point<$t> {
            pub fn distance_value(&self) -> $t {
                if !self.is_negative() {
                    return self.x + self.y;
                } else {
                    let mut p = self.clone();
                    p.abs();
                    p.x + p.y
                }
            }
        }
    };
}